#!/usr/bin/env python
# encoding: utf-8

from contextlib import contextmanager
import six
import logging

logger = logging.getLogger(__name__)


@contextmanager
def catch(errors=Exception, reraise=None, callback=logger.warning):
    """
    A context manager to catch the exceptions raised from block.
    :param errors: exception or exceptions tuple
    :param reraise: reraise a new exception from catched exception(default: None)
    :param callback: callback when catched a exception(default: logger.warning)
    """
    try:
        yield
    except errors as err:
        if callback:
            callback(err)
        if reraise:
            six.raise_from(reraise, err)


@contextmanager
def subprocessor(*args, **kwg):
    """
    With new subprocess call,terminate it when exit context
    :param args: argument pass to subprocess.Popen
    :param kwg: key word argument pass to subprocess.Popen
    """
    from subprocess import Popen
    processor = None
    try:
        processor = Popen(*args, **kwg)
        yield processor
    finally:
        if processor.poll() is None:
            processor.terminate()
            processor.wait()



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase
from collections import namedtuple

from ycyc.base import calcutils

ObjModel = namedtuple("ObjModel", ["key"])


class TestSafeCalc(TestCase):
    def test_usage(self):
        calc = calcutils.SafeCalc({
            "vint": 1, "vfloat": 2.3, "vstr": "456", "vlist": [7, 8.9, "10"],
            "vdict": {"key1": 11, "key2": ObjModel(key=12.13)}
        })
        self.assertEqual(calc("vint+1"), 2)
        self.assertEqual(calc("0.09 < vfloat/23 < 0.1"), True)
        self.assertEqual(calc("vstr == '456'"), True)
        self.assertEqual(calc("7 in vlist"), True)
        self.assertEqual(set(calc("vdict.keys()")), set(["key1", "key2"]))
        self.assertEqual(calc("vdict['key1']/10 == vint"), True)
        self.assertEqual(calc("12 and 34 or 56"), 34)

    def test_func(self):
        import operator

        calc = calcutils.SafeCalc({
            "add": operator.add, "sub": operator.sub,
        })

        self.assertEqual(calc("add(1, sub(2, 3))"), 0)

        with self.assertRaises(calcutils.NameParseError):
            calc("mul(4, 5)")

        with self.assertRaises(NameError):
            calc("mul")

        self.assert_(issubclass(calcutils.NameParseError, NameError))

    def test_evil(self):
        forbidden_exprs = (
            "import os", "open('test.txt', 'w')", "def x(): pass; print x.func_globals",
            "().__class__.mro()[1].__subclasses__()"
        )
        calc = calcutils.SafeCalc({})

        for e in forbidden_exprs:
            with self.assertRaises(Exception):
                print e
                calc(e)



#!/usr/bin/env python
# encoding: utf-8

"""
A adapter tool module
"""

import collections


class ObjAsDictAdapter(collections.Mapping):
    """
    Provided a adapter that allow visit a object as dict
    """
    def __init__(self, obj):
        self.__object = obj

    def __getitem__(self, name):
        try:
            return getattr(self.__object, name)
        except AttributeError:
            raise KeyError(name)

    def __setitem__(self, name, value):
        setattr(self.__object, name, value)

    def __iter__(self):
        return iter(dir(self.__object))

    def __len__(self):
        return len(dir(self.__object))



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

from ycyc.base import adapter


class TestObjAsDictAdapter(TestCase):
    class Parent(object):
        pclsval = 1

        def __init__(self):
            self.pinsval = 2

    class Child(Parent):
        cclsval = 3

        def __init__(self):
            super(TestObjAsDictAdapter.Child, self).__init__()
            self.cinsval = 4

        def __getattr__(self, name):
            if name == "dynamicval":
                return 5

    def test_base_method(self):
        parent = self.Parent()
        parent_dict = adapter.ObjAsDictAdapter(parent)

        self.assertEqual(parent_dict["pclsval"], parent.pclsval)
        self.assertEqual(parent_dict["pinsval"], parent.pinsval)
        self.assertIn("pclsval", iter(parent_dict))
        self.assertIn("pinsval", iter(parent_dict))

    def test_inherit(self):
        child = self.Child()
        child_dict = adapter.ObjAsDictAdapter(child)

        self.assertEqual(child_dict["pclsval"], child.pclsval)
        self.assertEqual(child_dict["pinsval"], child.pinsval)
        self.assertEqual(child_dict["cclsval"], child.cclsval)
        self.assertEqual(child_dict["cinsval"], child.cinsval)
        self.assertIn("pclsval", iter(child_dict))
        self.assertIn("pinsval", iter(child_dict))
        self.assertIn("cclsval", iter(child_dict))
        self.assertIn("cinsval", iter(child_dict))

    def test_dynamic_attr(self):
        child = self.Child()
        child_dict = adapter.ObjAsDictAdapter(child)

        self.assertEqual(child_dict["dynamicval"], child.dynamicval)
        self.assertNotIn("dynamicval", iter(child_dict))

        child.dynamicval = 6
        self.assertEqual(child_dict["dynamicval"], child.dynamicval)
        self.assertIn("dynamicval", iter(child_dict))

    def test_dict_interface(self):
        child = self.Child()
        parent = self.Parent()
        child_dict = adapter.ObjAsDictAdapter(child)
        parent_dict = adapter.ObjAsDictAdapter(parent)

        self.assertEqual(parent_dict.get("cinsval"), None)
        self.assertEqual(
            parent_dict.get("cinsval", child_dict["cinsval"]),
            child_dict.get("cinsval"))

        self.assertGreater(len(parent_dict), 0)
        self.assertEqual(parent_dict.keys(), list(parent_dict.iterkeys()))
        self.assertEqual(parent_dict.values(), list(parent_dict.itervalues()))
        self.assertEqual(parent_dict.items(), list(parent_dict.iteritems()))
        self.assertEqual(len(parent_dict), len(list(parent_dict)))



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

from ycyc.base import funcutils


class TestCoding(TestCase):
    b_en = b"test"
    b_ch = u"测试".encode("utf-8")
    u_en = u"test"
    u_ch = u"测试"

    def test_encode(self):
        self.assertEqual(self.b_en, funcutils.encode(self.u_en, "utf-8"))
        self.assertEqual(self.b_ch, funcutils.encode(self.u_ch, "utf-8"))
        self.assertEqual(self.b_en, funcutils.encode(self.b_en, "utf-8"))
        self.assertEqual(self.b_ch, funcutils.encode(self.b_ch, "utf-8"))

    def test_decode(self):
        self.assertEqual(self.u_en, funcutils.decode(self.u_en, "utf-8"))
        self.assertEqual(self.u_ch, funcutils.decode(self.u_ch, "utf-8"))
        self.assertEqual(self.u_en, funcutils.decode(self.b_en, "utf-8"))
        self.assertEqual(self.u_ch, funcutils.decode(self.b_ch, "utf-8"))

    def test_bytescat(self):
        target = u"test测试".encode("utf-8")
        self.assertEqual(funcutils.bytescat(self.b_en, self.b_ch), target)
        self.assertEqual(funcutils.bytescat(self.u_en, self.u_ch), target)
        self.assertEqual(funcutils.bytescat(self.b_en, self.u_ch), target)
        self.assertEqual(funcutils.bytescat(self.u_en, self.b_ch), target)

        target = u"测试测试".encode("utf-8")
        self.assertEqual(funcutils.bytescat(self.b_ch, self.b_ch), target)
        self.assertEqual(funcutils.bytescat(self.u_ch, self.u_ch), target)
        self.assertEqual(funcutils.bytescat(self.b_ch, self.u_ch), target)
        self.assertEqual(funcutils.bytescat(self.u_ch, self.b_ch), target)

    def test_strcat(self):
        target = u"test测试"
        self.assertEqual(funcutils.strcat(self.b_en, self.b_ch), target)
        self.assertEqual(funcutils.strcat(self.u_en, self.u_ch), target)
        self.assertEqual(funcutils.strcat(self.b_en, self.u_ch), target)
        self.assertEqual(funcutils.strcat(self.u_en, self.b_ch), target)

        target = u"测试测试"
        self.assertEqual(funcutils.strcat(self.b_ch, self.b_ch), target)
        self.assertEqual(funcutils.strcat(self.u_ch, self.u_ch), target)
        self.assertEqual(funcutils.strcat(self.b_ch, self.u_ch), target)
        self.assertEqual(funcutils.strcat(self.u_ch, self.b_ch), target)

        gbk_ch = u"测试".encode("gbk")
        self.assertEqual(
            funcutils.strcat(gbk_ch, self.b_ch, "gbk", "utf-8"), target)
        self.assertEqual(
            funcutils.strcat(gbk_ch, self.u_ch, "gbk", "utf-8"), target)
        self.assertEqual(
            funcutils.strcat(self.b_ch, gbk_ch, "utf-8", "gbk"), target)
        self.assertEqual(
            funcutils.strcat(self.u_ch, gbk_ch, "utf-8", "gbk"), target)



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

from ycyc.base.contextutils import catch


class TestCatch(TestCase):
    def test_catch_errors(self):
        with catch():
            raise Exception()

        with catch(ZeroDivisionError):
            1 / 0

        with catch((KeyError, IndexError)):
            list()[1]
            dict()["nothing"]

    def test_raise(self):
        with self.assertRaises(KeyError):
            with catch(IndexError):
                dict()["nothing"]

        with self.assertRaises(IndexError):
            with catch(KeyError, reraise=IndexError):
                dict()["nothing"]

        with self.assertRaises(ValueError):
            with catch((ZeroDivisionError, IndexError), reraise=ValueError):
                1 / 0

        with self.assertRaises(ValueError):
            with catch((ZeroDivisionError, IndexError), reraise=ValueError):
                list()[1]

        with self.assertRaises(ValueError):
            try:
                valerr = ValueError()
                with catch(ZeroDivisionError, reraise=valerr):
                    1 / 0

            except ValueError as err:
                self.assertIs(err, valerr)
                raise

    def test_callback(self):
        results = []

        with catch((ZeroDivisionError, IndexError), callback=results.append):
            1 / 0
        self.assertIsInstance(results.pop(), ZeroDivisionError)

        with catch((ZeroDivisionError, IndexError), callback=results.append):
            list()[1]
        self.assertIsInstance(results.pop(), IndexError)
